;;; ignore-window.el

(defvar ignored-windows '())

(defun window-ignored-p (&optional window)
  (member (or window (selected-window)) ignored-windows))

(defun ignore-window (&optional window)
  (interactive)
  (if (not window)
      (setq window (selected-window)))
  (unless (window-ignored-p window)
    (setq ignored-windows (cons window ignored-windows))))

(defun unignore-window (&optional window)
  (interactive)
  (if (not window)
      (setq window (selected-window)))
  (when (window-ignored-p window)
    (setq ignored-windows (remove window ignored-windows))))

(defun toggle-window-ignore (&optional window)
  (interactive)
  (if (not window)
      (setq window (selected-window)))
  (cond ((window-ignored-p window)
         (message "Unignoring window")
         (unignore-window window))
        (t
         (message "Ignoring window")
         (ignore-window window))))


(defadvice delete-window (after ignore-window activate)
  (when (window-ignored-p (selected-window))
    (other-window 1)))

(defadvice other-window (after ignore-window activate)
  (when (window-ignored-p (selected-window))
    (other-window 1 all-frames)))

(defun other-ignored-window ()
  (interactive)
  (let ((window (next-window (selected-window))))
    (while (and (not (eq window (selected-window)))
                (not (window-ignored-p window)))
      (setq window (next-window window)))
    (select-window window)))

(global-set-key "\C-ci" 'toggle-window-ignore)
(global-set-key "\C-co" 'other-ignored-window)

(provide 'ignore-window)
