;;; lock-window.el

(defvar locked-windows '())

(defun window-locked-p (&optional window)
  (member (or window (selected-window)) locked-windows))

(defun cleanup-locked-windows ()
  "Remove dead windows from locked-windows"
  (let ((old locked-windows))
    (setq locked-windows)
    (while old
      (when (window-live-p (car old))
        (setq locked-windows (cons (car old) locked-windows)))
      (setq old (cdr old)))))
                

(defun lock-window (&optional window)
  (interactive)
  (if (not window)
      (setq window (selected-window)))
  (unless (window-locked-p window)
    (set-window-dedicated-p window t)
    (setq locked-windows (cons window locked-windows))))

(defun unlock-window (&optional window)
  (interactive)
  (if (not window)
      (setq window (selected-window)))
  (when (window-locked-p window)
    (set-window-dedicated-p window nil)
    (setq locked-windows (remove window locked-windows))))

(defun toggle-window-lock (&optional window)
  (interactive)
  (cleanup-locked-windows)
  (if (not window)
      (setq window (selected-window)))
  (cond ((window-locked-p window)
         (message "Unlocking window")
         (unlock-window window))
        (t
         (message "Locking window")
         (lock-window window))))

(defadvice delete-window (before lock-window activate)
  (cleanup-locked-windows)
  (if (window-locked-p window)
      (unlock-window window)
    (when (< (- (length (window-list))
                (length locked-windows))
             2)
      (error "Only locked windows left"))))

(defadvice delete-other-windows (around lock-window activate)
  (cleanup-locked-windows)
  (let ((win (or window (selected-window))))
    (cond ((window-locked-p win)
           (error "This window is locked"))

          (locked-windows
           (save-selected-window
             (let ((windows (remove win (window-list))))
               (while windows
                 (unless (window-locked-p (car windows))
                   (delete-window (car windows)))
                 (setq windows (cdr windows))))))

          (t
           ad-do-it))))

(global-set-key "\C-cl" 'toggle-window-lock)

(provide 'lock-window)
